--PetOverlay v1.0
-- Original macro code by Xelron @ Feathermoon ( http://forums.worldofwarcraft.com/thread.html?topicId=10971909982&postId=109708938639 )
-- Fixed and converted to plugin by Cybermind @ Stormrage
-- Sparkle code (to replace non-existent autocast overlay) from VendorBait by Tekkub @ http://tekkub.net/addons/VendorBait

-- Initialize PetOverlay
-- self = parent frame for addon
function PetOverlay_OnLoad(self)
	-- initialize overlays
	PetOverlay_InitBar("Action")
	PetOverlay_InitBar("MultiBarBottomLeft")
	PetOverlay_InitBar("MultiBarBottomRight")
	PetOverlay_InitBar("MultiBarRight")
	PetOverlay_InitBar("MultiBarLeft")

	-- register an event handler
	self:SetScript("OnEvent", PetOverlay_EventHandler)
	self:RegisterEvent("PET_BAR_UPDATE")

	-- register a hook for the action button update function
	hooksecurefunc("ActionButton_Update", PetOverlay_ActionButton_Update)

	-- register chat command
	SLASH_PETOVERLAY1 = "/petol";
	SlashCmdList["PETOVERLAY"] = PetOverlay_Slash;

	DEFAULT_CHAT_FRAME:AddMessage("PetOverlay v1.0 initialized! Type /petol for help", 0.66, 0.1, 0.1, 2385, 5)
end


-- slash command handler
function PetOverlay_Slash(s)
	DEFAULT_CHAT_FRAME:AddMessage("PetOverlay help:", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("To use, include the following in your macro:", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  #pet skillname", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("Where 'skillname' is the name of a skill, such as", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  #pet Call of the Wild", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("This will add an autocast overlay showing you the status", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("of the 'Call of the Wild' skill. To make a macro act like", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("a normal pet skill button, be sure to include a", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("/petautocasttoggle command on right click, such as:", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage(" ", 0, 0, 0, 0, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  #showtooltip Call of the Wild", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  #pet Call of the Wild", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  /petautocasttoggle [button:2] Call of the Wild", 0.9, 0.9, 0.9, 2385, 15)
	DEFAULT_CHAT_FRAME:AddMessage("  /cast [nobutton:2] Call of the Wild", 0.9, 0.9, 0.9, 2385, 15)
end


-- initialize texture and model for autocast overlays on each button of given bar
-- b = string corresponding to an action bar name
function PetOverlay_InitBar(b)
	local p, t, m, i

	for i = 1, 12 do
		-- texture is the yellow corners on box that pet skills have
		p = getglobal(b.."Button"..i)
		t = p:CreateTexture(b.."Button"..i.."AutoCastable", "OVERLAY")
		t:Hide()
		t:SetTexture("Interface/Buttons/UI-AutoCastableOverlay")
		t:SetPoint("CENTER")
		t:SetWidth(58)
		t:SetHeight(58)
		p["t"] = t

--		-- model is the actual sparkly border
--		m = CreateFrame("Model", b.."Button"..i.."AutoCast", p)
--		m:Hide()
--		m:ClearAllPoints()
--		m:SetAllPoints(p)
--		m:SetModel("Interface/Buttons/UI-AutoCastButton.mdx")
--		m:SetModelScale(1.2)
--		m:SetSequence(1)
--		m:SetSequenceTime(0, 0)
--		p["m"] = m
	end
end


-- called when an event happens
function PetOverlay_EventHandler(self, event)
	if (event == "PET_BAR_UPDATE") then
		-- update all action bars
		PetOverlay_UpdateBar("Action")
		PetOverlay_UpdateBar("MultiBarBottomLeft")
		PetOverlay_UpdateBar("MultiBarBottomRight")
		PetOverlay_UpdateBar("MultiBarRight")
		PetOverlay_UpdateBar("MultiBarLeft")
	end
end


--- generates a frame containing 4 gem socket sparkle textures and returns object
-- p = button that frame should be created under
function PetOverlay_MakeSparkleFrame(p)
	-- make a new frame
	local f = CreateFrame("Frame", nil, p)
	f:Hide()
	f:ClearAllPoints()
	f:SetPoint("TOPLEFT", p, "TOPLEFT", 2, -2)
	f:SetPoint("TOPRIGHT", p, "TOPRIGHT", -2, -2)
	f:SetPoint("BOTTOMLEFT", p, "BOTTOMLEFT", 2, 2)
	f:SetPoint("BOTTOMRIGHT", p, "BOTTOMRIGHT", -2, 2)


	-- load gem socket sparkle textures
	local ta = {}
	local i
	for i = 1, 12 do
		local t = f:CreateTexture(nil, "BACKGROUND")
		t:SetTexture("Interface/ItemSocketingFrame/UI-ItemSockets")
		t:SetTexCoord(0.3984375, 0.4453125, 0.40234375, 0.44921875)
		t:SetBlendMode("ADD")
		t:SetWidth(12 - (math.floor((i-1)/4) * 2))
		t:SetHeight(12 - (math.floor((i-1)/4) * 2))
		ta[i] = t
	end

	-- store textures under frame
	f["t"] = ta

	-- elapsed time
	f["e"] = { 0, 0, 0 }

	-- register show and update handlers
	f:SetScript("OnShow", function(self) self.e = { 0, 0, 0 } end)
	f:SetScript("OnUpdate", PetOverlay_SparkleUpdate)

	return f
end


-- called when sparkle frames update, do animations here
-- self = sparkle frame
-- elap = time elapsed since last update
function PetOverlay_SparkleUpdate(self, elap)
	local speed = 2
	local i

	for i = 1, 3 do
		self.e[i] = self.e[i] + elap

		if (self.e[i] > (speed * i)) then
			self.e[i] = 0
		end
	end
	 
	local distance = self:GetWidth()

	for i = 1, 3 do
		self.t[1+((i-1)*4)]:SetPoint("CENTER", self, "TOPLEFT", self.e[i] / (speed * i) * distance, 0)
		self.t[2+((i-1)*4)]:SetPoint("CENTER", self, "BOTTOMRIGHT", -self.e[i] / (speed * i) * distance, 0)
		self.t[3+((i-1)*4)]:SetPoint("CENTER", self, "TOPRIGHT", 0, -self.e[i] / (speed * i) * distance)
		self.t[4+((i-1)*4)]:SetPoint("CENTER", self, "BOTTOMLEFT", 0, self.e[i] / (speed * i) * distance)
	end
end


-- called after ActionButton_Update, this is where we show/hide the overlays
-- self = an individual action button object
function PetOverlay_ActionButton_Update(self)
	-- if this button doesn't have a corner texture created for it, ignore it (bonus bar buttons)
	if (not self.t) then
		return
	end

	local a = self.action
	local T, I = GetActionInfo(a)

	-- simple flags to decide whether to hide at the end of the function
	local t_show = 0
	local m_show = 0

	-- if this button is a macro
	if (HasAction(a) and T == "macro") then

		-- check for "#pet" line in macro
		local f, _, s = string.find(GetMacroBody(I), "#pet ([^\n]*)")

		-- if line is found
		if (f) then
			-- get allowed/enabled status of autocast
			local a, e = GetSpellAutocast(GetPetSpellID(s), BOOKTYPE_PET)

			-- if allowed to autocast (meaning pet), show corners
			if (a) then
				t_show = 1
				self.t:Show()
			end

			-- if autocast is enabled, show overlay sparkle textures
			if (e) then
				if (not self.m) then
					local f = PetOverlay_MakeSparkleFrame(self)
					self["m"] = f
				end

				m_show = 1
				self.m:Show()
			end

		end
	end

	-- default hide corners texture
	if (t_show == 0) then
		self.t:Hide()
	end

	-- default hide sparkle textures
	if (m_show == 0) then
		if (self.m) then
			self.m:Hide()
		end
	end
end


-- helper function to call update function for each button of given bar
-- b = string corresponding to an action bar name
function PetOverlay_UpdateBar(b)
	local i

	for i = 1, 12 do
		self = getglobal(b.."Button"..i)
		ActionButton_Update(self)
	end
end


-- helper function to get spell id for a pet spell name
-- s = string corresponding to a pet spell name
function GetPetSpellID(s)
	local i = 1

	while true do
		if (s == GetSpellName(i,BOOKTYPE_PET)) then
			return i
		elseif (not GetSpellName(i,BOOKTYPE_PET)) then
			return 99
		end

		i = i + 1
	end
end
